<?php

/**
 * @file
 * Generate a admin UI.
 */

/**
 * UI controller.
 */
class RulesLinkUIController extends EntityDefaultUIController {

  /**
   * Overrides hook_menu() defaults.
   */
  public function hook_menu() {
    $items = parent::hook_menu();
    $items[$this->path]['description'] = 'Manage links that trigger rules';
    return $items;
  }
}

/**
 * Generates the rules link editing form.
 */
function rules_link_form($form, &$form_state, $rules_link, $op = 'edit') {
  if ($op == 'clone') {
    $rules_link->label .= ' (cloned)';
    $rules_link->type = '';
  }

  $form['title'] = array(
    '#title' => t('Title'),
    '#type' => 'textfield',
    '#default_value' => isset($rules_link->title) ? $rules_link->title : '',
    '#description' => t('The human-readable name of this rules link'),
    '#required' => TRUE,
    '#size' => 30,
  );

  $form['name'] = array(
    '#type' => 'machine_name',
    '#description' => t('A unique machine-readable name for this rules link. It must only contain lowercase letters, numbers, and underscores.'),
    '#default_value' => isset($rules_link->name) ? $rules_link->name : '',
    '#disabled' => $rules_link->hasStatus(ENTITY_IN_CODE) && $op != 'clone',
    '#required' => TRUE,
    '#machine_name' => array(
      'exists' => 'rules_link_get_links',
      'source' => array('title'),
    ),
  );

  $form['link_type'] = array(
    '#type' => 'radios',
    '#title' => t('Link type'),
    '#default_value' => isset($rules_link->settings['link_type']) ? $rules_link->settings['link_type'] : 'token',
    '#options' => array('javascript' => t('Javascript'), 'token' => t('Normal'), 'confirm' => t('Confirmation')),
  );

  global $base_url;
  $form['path'] = array(
    '#title' => t('Path'),
    '#description' => t('(Optional) Path for the link. It not set a default link will be generated.'),
    '#default_value' => isset($rules_link->path) ? $rules_link->path : '',
    '#type' => 'textfield',
    '#field_prefix' => $base_url . '/',
  );

  $entity_options = array();
  $entities = entity_get_info();
  foreach ($entities as $key => $entity) {
    $entity_options[$key] = t($entity['label']);
  }

  $default_entity = isset($form_state['values']['entity_type']) ? $form_state['values']['entity_type'] : (isset($rules_link->entity_type) ? $rules_link->entity_type : 'node');

  $form['entity_type'] = array(
    '#title' => t('Entity Type'),
    '#description' => t('Enity type to which the link should be attached to.'),
    '#default_value' => $default_entity,
    '#options' => $entity_options,
    '#required' => TRUE,
    '#type' => 'select',
    '#maxlength' => 32,
    '#ajax' => array(
      'callback' => 'rules_link_bundle_js_callback',
      'wrapper' => 'rules-link-bundle',
      'speed' => 'fast',
    ),
  );

  $bundle_options = array();
  if (isset($form_state['clicked_button']) && $form_state['clicked_button']['#parents'][0] == 'show_all') {
    foreach ($entities as $key => $entity) {
      foreach ($entity['bundles'] as $key => $bundle) {
        $bundle_options[$key] = $bundle['label'];
      }
    }
  }
  else {
    foreach ($entities[$default_entity]['bundles'] as $key => $bundle) {
      $bundle_options[$key] = $bundle['label'];
    }
  }

  $form['bundle_wrapper'] = array(
    '#prefix' => '<div id="rules-link-bundle">',
    '#suffix' => '</div>',
  );

  $form['bundle_wrapper']['bundle'] = array(
    '#title' => t('Bundle'),
    '#description' => t('Bundle to which the link should be attached to.'),
    '#type' => 'select',
    '#options' => $bundle_options,
    '#default_value' => isset($rules_link->bundle) ? $rules_link->bundle : '',
    '#maxlength' => 32,
  );

  if ($default_entity == 'node') {
    $form['bundle_wrapper']['node_link'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show link in node.'),
      '#description' => t('If set, a link will be shown on each node to which the bundle and the visibility conditions apply.'),
      '#default_value' => isset($rules_link->settings['node_link']) ? $rules_link->settings['node_link'] : FALSE,
    );
  }

  $html_id_show_all= drupal_html_id('rules-link-show-all-wrapper');

  $form['show_all'] = array(
    '#type' => 'button',
    '#value' => t('Show All'),
    '#prefix' => "<div id='$html_id_show_all'>",
    '#suffix' => '</div>'
  );

  // Hide the button for users with javascript.
  drupal_add_js('(function ($) { $("#' . $html_id_show_all . '").hide() }(jQuery));',
    array('type' => 'inline', 'scope' => 'footer')
  );

  // Hide the rule overviews when creating a new link.
  if ($op != 'add') {
    $condition = rules_config_load($rules_link->settings['condition_set']);
    $condition->form($form['condition_set'], $form_state);
    $form['condition_set']['elements']['#caption'] = 'Visibility conditions';
    // Since we never want to negate the result, we can hide the form.
    unset($form['condition_set']['negate']);

    $rule_set = rules_config_load($rules_link->settings['rule_set']);
    $rule_set->form($form['rule_set'], $form_state);
    $form['rule_set']['elements']['#caption'] = 'Reactions';

    $form['system_message']['msg'] = array(
      '#markup' => t('To output a system message, simply use the "Show a message on the site" action in one of your rules. This will even work for javascript links.'),
      '#prefix' => '<i>',
      '#suffix' => '</i>',
    );
  }

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Rules Link'),
    '#weight' => 40,
  );

  return $form;
}

/**
 * Callback for the select box for bundles.
 */
function rules_link_bundle_js_callback($form, $form_state) {
  return $form['bundle_wrapper'];
}

/**
 * Validation form for editing form.
 */
function rules_link_form_validate(&$form, &$form_state) {
  $rules_link = entity_ui_form_submit_build_entity($form, $form_state);

  $path = empty($rules_link->path) ? 'rules_link/' . $rules_link->name : $rules_link->path;

  $menu_item = menu_get_item($rules_link->path);
  $lookup_path = drupal_lookup_path('source', $rules_link->path);
  if (!empty($menu_item) || !empty($lookup_path)) {
    form_error($form['path'], t('That path "%path" is already in use. This system cannot override existing paths.', array('%path' => $path)));  
  }

  foreach (rules_link_get_links() as $link) {
    if ($link->path == $path && $link->name != $rules_link->name) {
      form_error($form['path'], t('That path "%path" is taken by the rules link "%link". This system cannot override existing paths.',
        array(
          '%path' => $path,
          '%link' => $link->name,
        )
      ));
    }
  }
}

/**
 * Form API submit callback for the type form.
 */
function rules_link_form_submit(&$form, &$form_state) {
  $rules_link = entity_ui_form_submit_build_entity($form, $form_state);
  $form_state['redirect'] = 'admin/config/workflow/rules_links';

  // If its a new link, we need to create a new rule set and redirect to the
  // edit page.
  if (!isset($rules_link->settings['rule_set']) && !isset($rules_link->settings['condition_set'])) {
    $rules_link->settings['condition_set'] = 'rules_link_condition_' . $rules_link->name;
    $conditions = rules_and(array(
      'entity' => array('type' => $form_state['values']['entity_type'], 'label' => $form_state['values']['entity_type']),
    ));
    $conditions->save($rules_link->settings['condition_set'], 'rules_link');

    $rules_link->settings['rule_set'] = 'rules_link_set_' . $rules_link->name;
    $rule_set = rules_rule_set(array(
      'entity' => array('type' => $form_state['values']['entity_type'], 'label' => $form_state['values']['entity_type']),
    ));
    $rule_set->save($rules_link->settings['rule_set'], 'rules_link');

    $form_state['redirect'] = 'admin/config/workflow/rules_links/manage/' . $rules_link->name;
  }

  if (empty($rules_link->path)) {
    $rules_link->path = $rules_link->name;
  }

  $rules_link->settings['link_type'] = $form_state['values']['link_type'];
  $rules_link->settings['node_link'] = isset($form_state['values']['node_link']) ? $form_state['values']['node_link'] : FALSE;

  // Save and go back.
  $rules_link->save();
  menu_rebuild();
}

/**
 * Implements hook_rules_link_delete to delete the condition and rule set.
 */
function rules_link_rules_link_delete(RulesLink $rules_link) {
  rules_config_load($rules_link->settings['condition_set'])->delete();
  rules_config_load($rules_link->settings['rule_set'])->delete();
}